#ifndef VULCAN_UI_H_
#define VULCAN_UI_H_

struct game_context;

struct timer {
	unsigned long start_msec;
	unsigned long timeout_msec;
	void *extra;
	int one_shot;
	int (*action)(struct game_context *, void *);
	struct timer *next;
};

enum computer_player_state {
	IDLE,
	WAITING_FOR_USER_INPUT,
	ANIMATING,
	THINKING,
	DONE_MATE_FOR_BLACK,
	DONE_MATE_FOR_WHITE,
};

struct game_context {
	struct game_state state;
	struct selected_squares selected_squares;

	int cur_side;
	int max_depth;				/* max AI search depth */
	enum computer_player_state computer_player_state;
	enum player_type players[2];
	char last_move_buf[120];

	struct matrix rmatrix;			/* rotation matrix */
	struct vector board_pos;		/* translation */

	struct position last_selected_pos;	/* last square selected by
						 * user  */

	int left_button_down;
	int right_button_down;
	int control_key_down;
	int start_position_selected;

	struct position last_highlighted_pos;
	int pos_highlighted;

	int last_clicked_x;
	int last_clicked_y;
	unsigned long last_click_msecs;

	struct timer *timer_list;
};

enum {
	BTN_LEFT = 1,
	BTN_RIGHT,
};

void
on_worker_reply(struct game_context *ctx, union move *move);

void
on_motion_notify(struct game_context *ctx, int button, int x, int y);

void
on_button_press(struct game_context *ctx, int button, int x, int y);

void
on_button_release(struct game_context *ctx, int button, int x, int y);

void
redraw(struct game_context *ctx);

void
reshape(struct game_context *ctx, int width, int height);

void
initialize_ui(struct game_context *ctx, enum player_type white_player,
  enum player_type black_player, int max_depth);

void
initialize_gl(struct game_context *ctx);

void
on_idle(struct game_context *ctx);

void
add_timer(struct game_context *ctx, unsigned long timeout_msec, int one_shot,
  int(*action)(struct game_context *, void *), void *extra);

#endif /* VULCAN_UI_H_ */
